import {
    createContext,
    type Resource,
    type Component,
    type ParentProps,
    createResource,
    useContext,
} from "solid-js";
import { getUserImages } from "../network";

type ImageWithRawData = Awaited<ReturnType<typeof getUserImages>>[number] & {
    rawData: string[];
};

type SearchImageStore = {
    images: Resource<ImageWithRawData[]>;
    onRefetchImages: () => void;
};

// How wonderfully functional
const getAllValues = (object: object): Array<string> => {
    const loop = (acc: Array<string>, next: object): Array<string> => {
        for (const _value of Object.values(next)) {
            const value: unknown = _value;
            switch (typeof value) {
                case "object":
                    if (value != null) {
                        acc.push(...loop(acc, value));
                    }
                    break;
                case "string":
                case "number":
                case "boolean":
                    acc.push(value.toString());
                    break;
                default:
                    break;
            }
        }

        return acc;
    };

    return loop([], object);
};

// On fast connections (or locally), the DB will send the complete update before the image has been fully saved.
// This is because it shouldn't really be an update, and that is hacky but hey. It works.
// The tech debt will be collected eventually.
//
// AND MORE IMPORTANTLY. The fucking AI agents aren't done.
// hhhhhhhhhhhhhhhhhhhhmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmmm
// wait groups.
const BIG_SHAME_TIMEOUT = 5000;

const SearchImageContext = createContext<SearchImageStore>();
export const SearchImageContextProvider: Component<ParentProps> = (props) => {
    const [images, { refetch }] = createResource(() =>
        getUserImages().then((data) => {
            return data.map((d) => ({
                ...d,
                rawData: getAllValues(d),
            }));
        }),
    );

    return (
        <SearchImageContext.Provider
            value={{
                images,
                onRefetchImages: () => setTimeout(refetch, BIG_SHAME_TIMEOUT),
            }}
        >
            {props.children}
        </SearchImageContext.Provider>
    );
};

export const useSearchImageContext = () => {
    const context = useContext(SearchImageContext);
    if (context == null) {
        throw new Error(
            "Unreachable: We should always have a mounted context and no undefined values",
        );
    }

    return context;
};
